import { Module } from '@nestjs/common';
import { RabbitMQModule } from '@golevelup/nestjs-rabbitmq';
import { ConfigModule, ConfigService } from '@nestjs/config';
import { WinstonModule } from 'nest-winston';
import * as winston from 'winston';
import { AdController } from './interface/message-controllers/ad.controller';
import { AuthController } from './interface/message-controllers/auth.controller';
import { UserController } from './interface/message-controllers/user.controller';
import { ConfigurationController } from './interface/message-controllers/configuration.controller';
import { MatcherController } from './interface/message-controllers/matcher.controller';
import { TerritoryController } from './interface/message-controllers/territory.controller';
import { GatewayApiController } from './interface/message-controllers/gateway-api.controller';
import { AdminApiController } from './interface/message-controllers/admin-api.controller';

@Module({
  imports: [
    RabbitMQModule.forRootAsync(RabbitMQModule, {
      imports: [ConfigModule],
      useFactory: async (configService: ConfigService) => ({
        exchanges: [
          {
            name: configService.get<string>('RMQ_EXCHANGE'),
            type: 'topic',
          },
        ],
        handlers: {
          loggingGatewayApiHealthCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.gateway-api.health.crit',
            queue: 'logging-gateway-api-health-crit',
          },
          loggingAdminApiHealthCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.admin-api.health.crit',
            queue: 'logging-admin-api-health-crit',
          },
          loggingAdCreatedInfo: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.ad.created.info',
            queue: 'logging-ad-created-info',
          },
          loggingAdCreatedWarning: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.ad.created.warning',
            queue: 'logging-ad-created-warning',
          },
          loggingAdCreatedCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.ad.created.crit',
            queue: 'logging-ad-created-crit',
          },
          loggingAdDeletedInfo: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.ad.deleted.info',
            queue: 'logging-ad-deleted-info',
          },
          loggingAdDeletedCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.ad.deleted.crit',
            queue: 'logging-ad-deleted-crit',
          },
          loggingAdReadWarning: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.ad.read.warning',
            queue: 'logging-ad-read-warning',
          },
          loggingAdUpdatedInfo: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.ad.updated.info',
            queue: 'logging-ad-updated-info',
          },
          loggingAdUpdatedCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.ad.updated.crit',
            queue: 'logging-ad-updated-crit',
          },
          loggingAdHealthCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.ad.health.crit',
            queue: 'logging-ad-health-crit',
          },
          loggingAuthCreatedCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.auth.created.crit',
            queue: 'logging-auth-created-crit',
          },
          loggingAuthDeletedCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.auth.deleted.crit',
            queue: 'logging-auth-deleted-crit',
          },
          loggingAuthReadWarning: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.auth.read.warning',
            queue: 'logging-auth-read-warning',
          },
          loggingAuthUsernameCreatedWarning: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.auth.username.created.warning',
            queue: 'logging-auth-username-created-warning',
          },
          loggingAuthUsernameDeletedWarning: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.auth.username.deleted.warning',
            queue: 'logging-auth-username-deleted-warning',
          },
          loggingAuthUsernameReadWarning: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.auth.username.read.warning',
            queue: 'logging-auth-username-read-warning',
          },
          loggingAuthPasswordUpdatedWarning: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.auth.password.updated.warning',
            queue: 'logging-auth-password-updated-warning',
          },
          loggingAuthUsernameUpdatedWarning: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.auth.username.updated.warning',
            queue: 'logging-auth-username-updated-warning',
          },
          loggingAuthHealthCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.auth.health.crit',
            queue: 'logging-auth-health-crit',
          },
          loggingConfigurationCreatedInfo: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.configuration.created.info',
            queue: 'logging-configuration-created-info',
          },
          loggingConfigurationCreatedWarning: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.configuration.created.warning',
            queue: 'logging-configuration-created-warning',
          },
          loggingConfigurationCreatedCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.configuration.created.crit',
            queue: 'logging-configuration-created-crit',
          },
          loggingConfigurationDeletedInfo: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.configuration.deleted.info',
            queue: 'logging-configuration-deleted-info',
          },
          loggingConfigurationDeletedCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.configuration.deleted.crit',
            queue: 'logging-configuration-deleted-crit',
          },
          loggingConfigurationReadWarning: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.configuration.read.warning',
            queue: 'logging-configuration-read-warning',
          },
          loggingConfigurationUpdatedInfo: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.configuration.updated.info',
            queue: 'logging-configuration-updated-info',
          },
          loggingConfigurationUpdatedCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.configuration.updated.crit',
            queue: 'logging-configuration-updated-crit',
          },
          loggingConfigurationHealthCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.configuration.health.crit',
            queue: 'logging-configuration-health-crit',
          },
          loggingMatcherAdCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.matcher.ad.crit',
            queue: 'logging-matcher-ad-crit',
          },
          loggingMatcherMatchCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.matcher.match.crit',
            queue: 'logging-matcher-match-crit',
          },
          loggingTerritoryCreatedInfo: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.territory.created.info',
            queue: 'logging-territory-created-info',
          },
          loggingTerritoryCreatedWarning: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.territory.created.warning',
            queue: 'logging-territory-created-warning',
          },
          loggingTerritoryCreatedCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.territory.created.crit',
            queue: 'logging-territory-created-crit',
          },
          loggingTerritoryDeletedInfo: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.territory.deleted.info',
            queue: 'logging-territory-deleted-info',
          },
          loggingTerritoryDeletedCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.territory.deleted.crit',
            queue: 'logging-territory-deleted-crit',
          },
          loggingTerritoryReadWarning: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.territory.read.warning',
            queue: 'logging-territory-read-warning',
          },
          loggingTerritoryUpdatedInfo: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.territory.updated.info',
            queue: 'logging-territory-updated-info',
          },
          loggingTerritoryUpdatedCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.territory.updated.crit',
            queue: 'logging-territory-updated-crit',
          },
          loggingTerritoryHealthCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.territory.health.crit',
            queue: 'logging-territory-health-crit',
          },
          loggingUserCreatedInfo: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.user.created.info',
            queue: 'logging-user-created-info',
          },
          loggingUserCreatedWarning: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.user.created.warning',
            queue: 'logging-user-created-warning',
          },
          loggingUserCreatedCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.user.created.crit',
            queue: 'logging-user-created-crit',
          },
          loggingUserDeletedInfo: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.user.deleted.info',
            queue: 'logging-user-deleted-info',
          },
          loggingUserDeletedCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.user.deleted.crit',
            queue: 'logging-user-deleted-crit',
          },
          loggingUserReadWarning: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.user.read.warning',
            queue: 'logging-user-read-warning',
          },
          loggingUserUpdatedInfo: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.user.updated.info',
            queue: 'logging-user-updated-info',
          },
          loggingUserUpdatedCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.user.updated.crit',
            queue: 'logging-user-updated-crit',
          },
          loggingUserHealthCrit: {
            exchange: configService.get<string>('RMQ_EXCHANGE'),
            routingKey: 'logging.user.health.crit',
            queue: 'logging-user-health-crit',
          },
        },
        uri: configService.get<string>('RMQ_URI'),
        connectionInitOptions: { wait: false },
        enableControllerDiscovery: true,
      }),
      inject: [ConfigService],
    }),
    WinstonModule.forRoot({
      levels: winston.config.syslog.levels,
      transports: [],
    }),
  ],
  controllers: [
    AdController,
    AuthController,
    UserController,
    ConfigurationController,
    MatcherController,
    TerritoryController,
    GatewayApiController,
    AdminApiController,
  ],
  providers: [],
  exports: [],
})
export class LoggerModule {}
